<?php

/**
 * Application.
 *
 * @author Katsuhiro Ogawa
 */
abstract class Application
{
    protected $debug = false;
    protected $request;
    protected $response;
    protected $session;
    protected $db_manager;
    protected $router;

    /**
     * Constructor.
     *
     * @param boolean $debug
     */
    public function __construct(bool $debug = false)
    {
        $this->setDebugMode($debug);
        $this->initialize();
        $this->configure();
    }

    /**
     * Sets debug mode.
     *
     * @param boolean $debug
     */
    protected function setDebugMode(bool $debug): void
    {
        $this->debug = (bool)$debug;

        if ($this->debug) {
            ini_set('display_errors', 1);
            error_reporting(-1);
        } else {
            ini_set('display_errors', 0);
        }
    }

    /**
     * Initializes the application.
     */
    protected function initialize(): void
    {
        $this->request    = new Request();
        $this->response   = new Response();
        $this->session    = new Session();
        $this->db_manager = new DbManager();
        $this->router     = new Router($this->registerRoutes());
    }

    /**
     * Configures the application.
     */
    protected function configure(): void
    {
    }

    /**
     * Gets the absolute path to the project root directory.
     *
     * @return string
     */
    abstract public function getRootDir(): string;

    /**
     * Registers routes.
     *
     * @return array
     */
    abstract protected function registerRoutes(): array;

    /**
     * Checks if the debug mode is enabled.
     *
     * @return boolean
     */
    public function isDebugMode(): bool
    {
        return $this->debug;
    }

    /**
     * Gets the request object.
     *
     * @return Request
     */
    public function getRequest(): Request
    {
        return $this->request;
    }

    /**
     * Gets the response object.
     *
     * @return Response
     */
    public function getResponse(): Response
    {
        return $this->response;
    }

    /**
     * Gets the session object.
     *
     * @return Session
     */
    public function getSession(): Session
    {
        return $this->session;
    }

    /**
     * Gets the database manager object.
     *
     * @return DbManager
     */
    public function getDbManager(): DbManager
    {
        return $this->db_manager;
    }

    /**
     * Gets the absolute path to the directory where the controller PHP files are stored.
     *
     * @return string
     */
    public function getControllerDir(): string
    {
        return $this->getRootDir() . '/controllers';
    }

    /**
     * Gets the absolute path to the directory where the view PHP files are stored.
     *
     * @return string
     */
    public function getViewDir(): string
    {
        return $this->getRootDir() . '/views';
    }

    /**
     * Gets the absolute path to the directory where the model PHP files are stored.
     *
     * @return string
     */
    public function getModelDir(): string
    {
        return $this->getRootDir() . '/models';
    }

    /**
     * Gets the absolute path to the directory where the web-accessible files are stored.
     *
     * @return string
     */
    public function getWebDir(): string
    {
        return $this->getRootDir() . '/web';