<?php

/**
 * Controller.
 *
 * @author Katsuhiro Ogawa <fivestar@nequal.jp>
 */
abstract class Controller
{
    protected string $controller_name;
    protected string $action_name;
    protected Application $application;
    protected Request $request;
    protected Response $response;
    protected Session $session;
    protected DbManager $db_manager;
    protected array $auth_actions = [];

    /**
     * Constructor.
     *
     * @param Application $application
     */
    public function __construct(Application $application)
    {
        $this->controller_name = strtolower(substr(get_class($this), 0, -10));

        $this->application = $application;
        $this->request = $application->getRequest();
        $this->response = $application->getResponse();
        $this->session = $application->getSession();
        $this->db_manager = $application->getDbManager();
    }

    /**
     * Runs an action.
     *
     * @param string $action
     * @param array $params
     * @return string The content to be returned as a response
     *
     * @throws UnauthorizedActionException if an unauthenticated user tries to access an action that requires authentication
     */
    public function run(string $action, array $params = []): string
    {
        $this->action_name = $action;

        $action_method = $action . 'Action';
        if (!method_exists($this, $action_method)) {
            $this->forward404();
        }

        if ($this->needsAuthentication($action) && !$this->session->isAuthenticated()) {
            throw new UnauthorizedActionException();
        }

        $content = $this->$action_method($params);

        return $content;
    }

    /**
     * Renders a view file.
     *
     * @param array $variables The associative array of variables to be passed to the template
     * @param string|null $template The name of the view file (if null, the action name is used)
     * @param string $layout The name of the layout file
     * @return string The content of the rendered view file
     */
    protected function render(array $variables = [], ?string $template = null, string $layout = 'layout'): string
    {
        $defaults = [
            'request' => $this->request,
            'base_url' => $this->request->getBaseUrl(),
            'session' => $this->session,
        ];

        $view = new View($this->application->getViewDir(), $defaults);

        if (is_null($template)) {
            $template = $this->action_name;
        }

        $path = $this->controller_name . '/' . $template;

        return $view->render($path, $variables, $layout);
    }

    /**
     * Throws a 404 error.
     *
     * @throws HttpNotFoundException
     */
    protected function forward404(): void
    {
        throw new HttpNotFoundException('Forwarded 404 page from ' . $this->controller_name . '/' . $this->action_name);
    }

    /**
     * Redirects to the specified URL.
     *
     * @param string $url
     */
    protected function redirect(string $url): void
    {
        if (!preg_match('#https?://#', $url)) {
            $protocol = $this->request->isSsl() ? 'https://' : 'http://';
            $host = $this->request->getHost();
            $base_url = $this->request->getBaseUrl();

            $url = $protocol . $host . $base_url