<?php

/**
 * Session.
 */
class Session
{
    protected static bool $sessionStarted = false;
    protected static bool $sessionIdRegenerated = false;

    public function __construct()
    {
        if (!self::$sessionStarted) {
            session_start();

            self::$sessionStarted = true;
        }
    }

    /**
     * Set a value in the session.
     *
     * @param string $name
     * @param mixed $value
     */
    public function set(string $name, mixed $value): void
    {
        $_SESSION[$name] = $value;
    }

    /**
     * Get a value from the session.
     *
     * @param string $name
     * @param mixed $default Default value to return if the key does not exist
     * @return mixed|null
     */
    public function get(string $name, mixed $default = null): mixed
    {
        return $_SESSION[$name] ?? $default;
    }

    /**
     * Remove a value from the session.
     *
     * @param string $name
     */
    public function remove(string $name): void
    {
        unset($_SESSION[$name]);
    }

    /**
     * Clear the session.
     */
    public function clear(): void
    {
        session_unset();
    }

    /**
     * Regenerate the session ID.
     *
     * @param bool $destroy Whether to destroy the old session data or not
     */
    public function regenerate(bool $destroy = true): void
    {
        if (!self::$sessionIdRegenerated) {
            session_regenerate_id($destroy);

            self::$sessionIdRegenerated = true;
        }
    }

    /**
     * Set the authentication status.
     *
     * @param bool $authenticated
     */
    public function setAuthenticated(bool $authenticated): void
    {
        $this->set('_authenticated', $authenticated);

        $this->regenerate();
    }

    /**
     * Check if the user is authenticated.
     *
     * @return bool
     */
    public function isAuthenticated(): bool
    {
        return $this->get('_authenticated', false);
    }
}