<?php

/**
 * Application.
 *
 * @author Katsuhiro Ogawa <fivestar@nequal.jp>
 */
abstract class Application
{
    protected bool $debug = false;
    protected Request $request;
    protected Response $response;
    protected Session $session;
    protected DbManager $db_manager;
    protected Router $router;

    /**
     * Constructor.
     *
     * @param bool $debug
     */
    public function __construct(bool $debug = false)
    {
        $this->setDebugMode($debug);
        $this->initialize();
        $this->configure();
    }

    /**
     * Set debug mode.
     *
     * @param bool $debug
     */
    protected function setDebugMode(bool $debug): void
    {
        $this->debug = $debug;
        ini_set('display_errors', $debug ? '1' : '0');
        error_reporting($debug ? -1 : 0);
    }

    /**
     * Initialize the application.
     */
    protected function initialize(): void
    {
        $this->request = new Request();
        $this->response = new Response();
        $this->session = new Session();
        $this->db_manager = new DbManager();
        $this->router = new Router($this->registerRoutes());
    }

    /**
     * Configure the application.
     */
    protected function configure(): void
    {
    }

    /**
     * Get the root directory of the project.
     *
     * @return string Absolute path to the root directory on the filesystem.
     */
    abstract public function getRootDir(): string;

    /**
     * Get routes.
     *
     * @return array
     */
    abstract protected function registerRoutes(): array;

    /**
     * Check if the debug mode is enabled.
     *
     * @return bool
     */
    public function isDebugMode(): bool
    {
        return $this->debug;
    }

    /**
     * Get Request object.
     *
     * @return Request
     */
    public function getRequest(): Request
    {
        return $this->request;
    }

    /**
     * Get Response object.
     *
     * @return Response
     */
    public function getResponse(): Response
    {
        return $this->response;
    }

    /**
     * Get Session object.
     *
     * @return Session
     */
    public function getSession(): Session
    {
        return $this->session;
    }

    /**
     * Get DbManager object.
     *
     * @return DbManager
     */
    public function getDbManager(): DbManager
    {
        return $this->db_manager;
    }

    /**
     * Get the path to the directory where controller files are stored.
     *
     * @return string
     */
    public function getControllerDir(): string
    {
        return $this->getRootDir() . '/controllers';
    }

    /**
     * Get the path to the directory where view files are stored.
     *
     * @return string
     */
    public function getViewDir(): string
    {
        return $this->getRootDir() . '/views';
    }

    /**
     * Get the path to the directory where model files are stored.
     *
     * @return string
     */
    public function getModelDir(): string
    {
        return $this->getRootDir() . '/models';
    }

    /**
     * Get the path to the document root.
     *
     * @return string
     */
    public function getWebDir(): string
    {
        return $this->getRootDir() . '/web';
    }

    /**
     * Run the application.
     *
     * @throws Http
